#include "grid.hpp"
#include "memorycard.hpp"
#include "block.hpp"
#include "icon_fetch.hpp"
#include <cassert>

namespace implem {
	void draw_icons (
		int icon_size,
		const MemoryCard& mc,
		std::vector<nana::animation>& icons,
		nana::widget& wid
	) {
		assert(icon_size);

		for (int z = 0; z < icons.size(); ++z) {
			nana::frameset fset;
			for (const auto& frame : icon_fetch(mc[z], icon_size, icon_size)) {
				nana::paint::image img;
				img.open(frame.data(), frame.size());
				fset.push_back(img);
			}

			icons[z].push_back(fset);
		}
	}
} //namespace implem

grid_drawer_trigger::grid_drawer_trigger() {
	std::cout << "constructing grid_drawer_trigger..." << std::endl;
	m_icons.reserve(15);
	for (int z = 0; z < 15; ++z) {
		m_icons.emplace_back(3);
		m_icons.back().looped(true);
		m_icons.back().play();
	}
}

void grid_drawer_trigger::attached (
	nana::widget& widget,
	nana::paint::graphics& graph
) {
	using nana::effects::edge_nimbus;
	using nana::API::dev::enable_space_click;
	using nana::API::tabstop;

	m_graph = &graph;
	m_widget = &widget;

	nana::window wd = widget;
	enable_space_click(widget, true);
	tabstop(wd);

	for (int z = 0; z < static_cast<int>(m_icons.size()); ++z) {
		const int x = (16 + 2) * (z % 3);
		const int y = (16 + 2) * (z / 3);
		m_icons[z].output(*m_widget, nana::point(x, y));
	}
}

void grid_drawer_trigger::detached() {
	std::cout << "grid_drawer_trigger::detach()" << std::endl;
	m_icons.clear();
	m_widget = nullptr;
	m_graph = nullptr;
	m_is_ms_down = false;
	m_icons_added = false;
}

void grid_drawer_trigger::set_memorycard (const MemoryCard* mc) {
	m_memorycard = mc;
}

//void grid_drawer_trigger::_m_draw_background (
//	nana::paint::graphics&,
//	bool is_mouse_down
//) {
//	assert(m_memorycard);
//	implem::draw_icons(16, *m_memorycard, m_icons, *m_widget);
//}


void grid_drawer_trigger::refresh (nana::paint::graphics& graph) {
	assert(m_widget);

	//draw background
	if (not nana::API::dev::copy_transparent_background(*m_widget, graph))
		graph.rectangle(true, m_widget->bgcolor());

	if (m_memorycard and not m_icons_added) {
		m_icons_added = true;
		implem::draw_icons(16, *m_memorycard, m_icons, *m_widget);
	}
}
